package elpanel;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.control.ToggleButton;
import javafx.scene.effect.ColorAdjust;
import javafx.scene.effect.DropShadow;
import javafx.scene.effect.Effect;
import javafx.scene.effect.ImageInput;
import javafx.scene.effect.Reflection;
import javafx.scene.image.Image;
import javafx.scene.text.Font;
import javafx.scene.text.FontPosture;

public class ElpanelController implements Initializable {
    public static final int port = 9999;

    private TextArea console;
    private Effect stand, stand_d;
    private ServerSocket serversocket;
    private Service<String> service;

    @FXML
    private ToggleButton tgl_stand;
    @FXML
    private ScrollPane scr_console;
    @FXML
    private Label lbl_version;

    @FXML
    public void handleTgl_stand(ActionEvent event) {
        boolean on = tgl_stand.isSelected();
        console.appendText("一般照明 GeneralLighting: " +
                           "reqSetOperationStatus(" + (on ? "ON" : "OFF") +
                           ") 送信\n");
        Elpanel.ArmadilloLighting(on);
    }

    @Override
    public void initialize(URL url, ResourceBundle rb) {
        lbl_version.setText("Java vendor: " +
                System.getProperty("java.vendor") + "\n" +
                "Java runtime version: " +
                System.getProperty("java.runtime.version") + "\n" +
                "Java VM version: " +
                System.getProperty("java.vm.version") + "\n" +
                "JavaFX runtime version: " +
                System.getProperty("javafx.runtime.version"));

        console = new TextArea();
        console.setFont(Font.font("SansSerif", FontPosture.REGULAR, 18));
        console.setEditable(false);
        scr_console.setContent(console);
        scr_console.setFitToWidth(true);
        scr_console.setFitToHeight(true);

        ImageInput imageinput;
        Reflection reflection;
        ColorAdjust coloradjust;
        DropShadow dropshadow;

        /* Armadillo照明ボタン用の画像を準備 */
        imageinput = new ImageInput(new Image("elpanel/stand.jpg"));
        reflection = new Reflection();
        reflection.setInput(imageinput);
        dropshadow = new DropShadow();
        dropshadow.setInput(reflection);
        stand = dropshadow;
        coloradjust = new ColorAdjust();
        coloradjust.setBrightness(-0.75);
        coloradjust.setInput(imageinput);
        dropshadow = new DropShadow();
        dropshadow.setInput(coloradjust);
        stand_d = dropshadow;

        try {
            /* Elpanelから状態変化を受信するためのソケットを準備 */
            serversocket = new ServerSocket(port);
            serversocket.setReuseAddress(true);
        } catch (IOException ex) {
            Logger.getLogger(ElpanelController.class.getName()).log(Level.SEVERE,
                             null, ex);
        }
        /* 状態変化受信用サービスを開始 */
        service = new Service<String>() {
            @Override
            protected Task<String> createTask() {
                Task<String> task = new Task<String>() {
                    @Override
                    protected String call() throws IOException {
                        Socket socket = serversocket.accept();
                        BufferedReader reader =
                                new BufferedReader(new InputStreamReader
                                                   (socket.getInputStream()));
                        return reader.readLine();
                    }
                };
                task.setOnSucceeded((WorkerStateEvent event) -> {
                    String value = event.getSource().getValue().toString();
                    /* 状態変化に応じてメッセージ表示&ボタン状態変更 */
                    if (value.equalsIgnoreCase("1: on")) {
                        console.appendText("一般照明 GeneralLighting: " +
                                           "reqGetOperationStatus(ON) 受信\n");
                        tgl_stand.setSelected(true);
                        tgl_stand.setEffect(stand);
                    } else if (value.equalsIgnoreCase("1: off")) {
                        console.appendText("一般照明 GeneralLighting: " +
                                           "reqGetOperationStatus(OFF) 受信\n");
                        tgl_stand.setSelected(false);
                        tgl_stand.setEffect(stand_d);
                    }
                    service.restart();
                });
                return task;
            }
        };
        service.start();
    }
}
